use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
AnnoCPAN
CPAN
Chaves
CheckCommit
CheckCommitAuthor
CheckCommitAuthorFromMailmap
CheckCommitBase
CheckIndent
CheckPerl
Gerrit
Git
GitRepoAdmin
GitWeb
Github
Gravatar
Gustavo
HTTPS
IO
JOBSUFFIX
JSON
Koivunalho
Leite
Mendonça
Mikko
MoreHooks
POSIX
Params
REFSPEC
Readonly
Storable
Subdirectory
TODO
Timestamp
TriggerJenkins
VCS
alnum
applypatch
ascii
aws
cas
committer
committers
de
dir
filename
githooks
gnustavo
http
https
io
jobname
json
lib
licensable
lv
mailmap
messager
mikkoi
msg
multi
params
patchset
runtime
scalability
semafores
shortlog
startup
subitem
subparam
succcessful
successfull
timestamp
tradename
unreserves
url
userenv
webhooks
wildcard
wildcards
